@@ -3,6 +3,8 @@ package AnyEvent::Redis::RipeRedis;
 use 5.006000;
 use strict;
 use warnings;
+use base qw( Exporter );
+
 use fields qw(
   host
   port
@@ -25,16 +27,40 @@ use fields qw(
   subs
 );
 
-our $VERSION = '1.009';
+our $VERSION = '1.100';
 
 use AnyEvent::Handle;
 use Encode qw( find_encoding is_utf8 );
 use Scalar::Util qw( looks_like_number weaken );
 use Carp qw( croak );
 
+BEGIN {
+  our @EXPORT_OK = qw( E_CANT_CONN E_LOADING_DATASET E_IO_OPERATION
+      E_CONN_CLOSED_BY_REMOTE_HOST E_CONN_CLOSED_ON_DEMAND E_NO_CONN
+      E_INVALID_PASS E_AUTH_REQUIRED E_COMMAND_EXEC E_UNEXPECTED );
+  our %EXPORT_TAGS = (
+    err_codes => \@EXPORT_OK,
+  );
+}
+
 use constant {
+  # Default values
   D_HOST => 'localhost',
   D_PORT => 6379,
+
+  # Error codes
+  E_CANT_CONN => 1,
+  E_LOADING_DATASET => 2,
+  E_IO_OPERATION => 3,
+  E_CONN_CLOSED_BY_REMOTE_HOST => 4,
+  E_CONN_CLOSED_ON_DEMAND => 5,
+  E_NO_CONN => 6,
+  E_INVALID_PASS => 7,
+  E_AUTH_REQUIRED => 8,
+  E_COMMAND_EXEC => 9,
+  E_UNEXPECTED => 10,
+
+  # String terminator
   EOL => "\r\n",
   EOL_LEN => 2,
 };
@@ -62,10 +88,7 @@ sub new {
 
   $self->{host} = $params->{host};
   $self->{port} = $params->{port};
-  if (
-    defined( $params->{password} ) and !ref( $params->{password} )
-      and $params->{password} ne ''
-      ) {
+  if ( defined( $params->{password} ) ) {
     $self->{password} = $params->{password};
     $self->{authed} = 0;
   }
@@ -92,9 +115,6 @@ sub new {
   return $self;
 }
 
-
-# Public method
-
 ####
 sub disconnect {
   my __PACKAGE__ $self = shift;
@@ -107,7 +127,7 @@ sub disconnect {
     $self->{authing} = 0;
     $self->{authed} = 0;
   }
-  $self->_abort_all( 'Connection closed by client' );
+  $self->_abort_all( 'Connection closed on demand', E_CONN_CLOSED_ON_DEMAND );
   if ( $was_connected and defined( $self->{on_disconnect} ) ) {
     $self->{on_disconnect}->();
   }
@@ -115,9 +135,6 @@ sub disconnect {
   return;
 }
 
-
-# Private methods
-
 ####
 sub _validate_new {
   my $params = pop;
@@ -158,8 +175,8 @@ sub _validate_new {
   }
   if ( !defined( $params->{on_error} ) ) {
     $params->{on_error} = sub {
-      my $err = shift;
-      warn "$err\n";
+      my $err_msg = shift;
+      warn "$err_msg\n";
     };
   }
   if ( !defined( $params->{on_connect_error} ) ) {
@@ -177,14 +194,14 @@ sub _connect {
   $self->{handle} = AnyEvent::Handle->new(
     connect => [ $self->{host}, $self->{port} ],
     keepalive => 1,
-    on_prepare => $self->_on_prepare(),
-    on_connect => $self->_on_connect(),
-    on_connect_error => $self->_on_connect_error(),
-    on_eof => $self->_on_eof(),
-    on_error => $self->_on_error(),
-    on_read => $self->_on_read(
+    on_prepare => $self->_create_on_prepare(),
+    on_connect => $self->_create_on_connect(),
+    on_connect_error => $self->_create_on_conn_error(),
+    on_eof => $self->_create_on_eof(),
+    on_error => $self->_create_on_error(),
+    on_read => $self->_create_on_read(
       sub {
-        return $self->_prcoess_response( @_ );
+        return $self->_on_response( @_ );
       }
     ),
   );
@@ -193,7 +210,7 @@ sub _connect {
 }
 
 ####
-sub _on_prepare {
+sub _create_on_prepare {
   my __PACKAGE__ $self = shift;
   weaken( $self );
 
@@ -207,7 +224,7 @@ sub _on_prepare {
 }
 
 ####
-sub _on_connect {
+sub _create_on_connect {
   my __PACKAGE__ $self = shift;
   weaken( $self );
 
@@ -226,207 +243,65 @@ sub _on_connect {
 }
 
 ####
-sub _on_connect_error {
+sub _create_on_conn_error {
   my __PACKAGE__ $self = shift;
   weaken( $self );
 
   return sub {
-    my $err = pop;
+    my $err_msg = pop;
 
     $self->{handle}->destroy();
     undef( $self->{handle} );
-    $err = "Can't connect to $self->{host}:$self->{port}: $err";
-    $self->_abort_all( $err );
-    $self->{on_connect_error}->( $err );
+    $err_msg = "Can't connect to $self->{host}:$self->{port}: $err_msg";
+    $self->_abort_all( $err_msg, E_CANT_CONN );
+    $self->{on_connect_error}->( $err_msg, E_CANT_CONN );
   };
 }
 
 ####
-sub _on_eof {
+sub _create_on_eof {
   my __PACKAGE__ $self = shift;
   weaken( $self );
 
   return sub {
-    $self->{handle}->destroy();
-    undef( $self->{handle} );
-    $self->{connected} = 0;
-    $self->{authing} = 0;
-    $self->{authed} = 0;
-    $self->_abort_all( 'Connection closed by remote host' );
-    if ( defined( $self->{on_disconnect} ) ) {
-      $self->{on_disconnect}->();
-    }
+    $self->_process_handle_error( 'Connection closed by remote host',
+        E_CONN_CLOSED_BY_REMOTE_HOST );
   };
 }
 
 ####
-sub _on_error {
+sub _create_on_error {
   my __PACKAGE__ $self = shift;
   weaken( $self );
 
   return sub {
-    my $err = pop;
-
-    $self->{handle}->destroy();
-    undef( $self->{handle} );
-    $self->{connected} = 0;
-    $self->{authing} = 0;
-    $self->{authed} = 0;
-    $self->_abort_all( $err );
-    $self->{on_error}->( $err );
-    if ( defined( $self->{on_disconnect} ) ) {
-      $self->{on_disconnect}->();
-    }
+    my $err_msg = pop;
+    $self->_process_handle_error( $err_msg, E_IO_OPERATION );
   };
 }
 
 ####
-sub _exec_command {
+sub _process_handle_error {
   my __PACKAGE__ $self = shift;
-  my $cmd_name = shift;
-  my @args = @_;
-  my $params = {};
-  if ( ref( $args[-1] ) eq 'HASH' ) {
-    $params = pop( @args );
-  }
+  my $err_msg = shift;
+  my $err_code = shift;
 
-  $params = $self->_validate_exec_cmd( $params );
-
-  my $cmd = {
-    name => $cmd_name,
-    args => \@args,
-    %{$params},
-  };
-
-  if ( exists( $SUB_ACTION_CMDS{$cmd->{name}} ) ) {
-    if ( $self->{sub_lock} ) {
-      $cmd->{on_error}->( "Command '$cmd->{name}' not allowed after 'multi' command."
-          . ' First, the transaction must be completed' );
-      return;
-    }
-    $cmd->{resp_remaining} = scalar( @{$cmd->{args}} );
-  }
-  elsif ( $cmd->{name} eq 'multi' ) {
-    $self->{sub_lock} = 1;
-  }
-  elsif ( $cmd->{name} eq 'exec' ) {
-    $self->{sub_lock} = 0;
-  }
-
-  if ( !defined( $self->{handle} ) ) {
-    if ( $self->{reconnect} ) {
-      $self->_connect();
-    }
-    else {
-      $cmd->{on_error}->( "Can't handle the command '$cmd->{name}'."
-          . ' No connection to the server' );
-      return;
-    }
-  }
-  if ( $self->{connected} ) {
-    if ( $self->{authed} ) {
-      $self->_push_to_handle( $cmd );
-    }
-    else {
-      if ( !$self->{authing} ) {
-        $self->_auth();
-      }
-      push( @{$self->{buffer}}, $cmd );
-    }
-  }
-  else {
-    push( @{$self->{buffer}}, $cmd );
+  $self->{handle}->destroy();
+  undef( $self->{handle} );
+  $self->{connected} = 0;
+  $self->{authing} = 0;
+  $self->{authed} = 0;
+  $self->_abort_all( $err_msg, $err_code );
+  $self->{on_error}->( $err_msg, $err_code );
+  if ( defined( $self->{on_disconnect} ) ) {
+    $self->{on_disconnect}->();
   }
 
   return;
 }
 
 ####
-sub _validate_exec_cmd {
-  my __PACKAGE__ $self = shift;
-  my $params = shift;
-
-  foreach my $name ( qw( on_done on_message on_error ) ) {
-    if (
-      defined( $params->{$name} )
-        and ref( $params->{$name} ) ne 'CODE'
-        ) {
-      croak "'$name' callback must be a code reference";
-    }
-  }
-
-  if ( !defined( $params->{on_error} ) ) {
-    $params->{on_error} = $self->{on_error};
-  }
-
-  return $params;
-}
-
-####
-sub _auth {
-  my __PACKAGE__ $self = shift;
-
-  $self->{authing} = 1;
-  $self->_push_to_handle( {
-    name => 'auth',
-    args => [ $self->{password} ],
-    on_done => sub {
-      $self->{authing} = 0;
-      $self->{authed} = 1;
-      $self->_flush_buffer();
-    },
-
-    on_error => sub {
-      my $err = shift;
-
-      $self->{authing} = 0;
-      $self->_abort_all( $err );
-      $self->{on_error}->( $err );
-    },
-  } );
-
-  return;
-}
-
-####
-sub _flush_buffer {
-  my __PACKAGE__ $self = shift;
-
-  my @commands = @{$self->{buffer}};
-  $self->{buffer} = [];
-  foreach my $cmd ( @commands ) {
-    $self->_push_to_handle( $cmd );
-  }
-
-  return;
-}
-
-####
-sub _push_to_handle {
-  my __PACKAGE__ $self = shift;
-  my $cmd = shift;
-
-  push( @{$self->{processing_queue}}, $cmd );
-  my $cmd_str = '';
-  my $m_bulk_len = 0;
-  foreach my $tkn ( $cmd->{name}, @{$cmd->{args}} ) {
-    if ( defined( $tkn ) ) {
-      if ( defined( $self->{encoding} ) and is_utf8( $tkn ) ) {
-        $tkn = $self->{encoding}->encode( $tkn );
-      }
-      my $tkn_len = length( $tkn );
-      $cmd_str .= "\$$tkn_len" . EOL . $tkn . EOL;
-      ++$m_bulk_len;
-    }
-  }
-  $cmd_str = "*$m_bulk_len" . EOL . $cmd_str;
-  $self->{handle}->push_write( $cmd_str );
-
-  return;
-}
-
-####
-sub _on_read {
+sub _create_on_read {
   my __PACKAGE__ $self = shift;
   my $cb = shift;
   weaken( $self );
@@ -500,11 +375,12 @@ sub _unshift_on_read {
   my $m_bulk_len = shift;
   my $cb = shift;
 
-  my $read_cb;
+  my $on_read;
   my @data;
   my @errors;
   my $remaining_num = $m_bulk_len;
-  my $cb_wrap = sub {
+
+  my $on_response = sub {
     my $data_chunk = shift;
     my $is_err = shift;
 
@@ -520,13 +396,13 @@ sub _unshift_on_read {
       ref( $data_chunk ) eq 'ARRAY' and @{$data_chunk}
         and $remaining_num > 0
         ) {
-      $hdl->unshift_read( $read_cb );
+      $hdl->unshift_read( $on_read );
     }
     elsif ( $remaining_num == 0 ) {
-      undef( $read_cb ); # Collect garbage
+      undef( $on_read ); # Collect garbage
       if ( @errors ) {
-        my $err = join( "\n", @errors );
-        $cb->( $err, 1 );
+        my $err_msg = join( "\n", @errors );
+        $cb->( $err_msg, 1 );
       }
       else {
         $cb->( \@data );
@@ -536,55 +412,107 @@ sub _unshift_on_read {
     }
   };
 
-  $read_cb = $self->_on_read( $cb_wrap );
-
-  $hdl->unshift_read( $read_cb );
+  $on_read = $self->_create_on_read( $on_response );
+  $hdl->unshift_read( $on_read );
 
   return;
 }
 
 ####
-sub _prcoess_response {
+sub _on_response {
   my __PACKAGE__ $self = shift;
   my $data = shift;
   my $is_err = shift;
 
-  my $cmd = $self->{processing_queue}[0];
-  if ( !defined( $cmd ) ) {
-    $self->{on_error}->( "Don't known how process response data."
-      . " Command queue is empty" );
-
-    return;
-  }
-
   if ( $is_err ) {
-    shift( @{$self->{processing_queue}} );
-    $cmd->{on_error}->( $data );
-
-    return;
+    $self->_process_error( $data );
+  }
+  elsif ( %{$self->{subs}} and $self->_is_pub_message( $data ) ) {
+    $self->_process_pub_message( $data );
+  }
+  else {
+    $self->_process_response( $data );
   }
 
-  if ( %{$self->{subs}} and $self->_is_sub_message( $data ) ) {
-    if ( exists( $self->{subs}{$data->[1]} ) ) {
-      $self->_process_sub_message( $data );
+  return;
+}
+
+####
+sub _process_response {
+  my __PACKAGE__ $self = shift;
+  my $data = shift;
+
+  my $cmd = $self->{processing_queue}[0];
+  if ( defined( $cmd ) ) {
+    if ( exists( $SUB_ACTION_CMDS{$cmd->{name}} ) ) {
+      $self->_process_sub_action( $cmd, $data );
       return;
     }
+    shift( @{$self->{processing_queue}} );
+    if ( $cmd->{name} eq 'quit' ) {
+      $self->disconnect();
+    }
+    if ( defined( $cmd->{on_done} ) ) {
+      $cmd->{on_done}->( $data );
+    }
+  }
+  else {
+    $self->{on_error}->( "Don't known how process response data."
+        . ' Command queue is empty', E_UNEXPECTED );
+  }
+
+  return;
+}
+
+####
+sub _process_pub_message {
+  my __PACKAGE__ $self = shift;
+  my $data = shift;
+
+  if ( exists( $self->{subs}{$data->[1]} ) ) {
+    my $sub = $self->{subs}{$data->[1]};
+    if ( exists( $sub->{on_message} ) ) {
+      if ( $data->[0] eq 'message' ) {
+        $sub->{on_message}->( $data->[1], $data->[2] );
+      }
+      else {
+        $sub->{on_message}->( $data->[2], $data->[3], $data->[1] );
+      }
+    }
+  }
+  else {
+    $self->{on_error}->( "Don't known how process published message."
+      . " Unknown channel or pattern '$data->[1]'", E_UNEXPECTED );
+  }
+
+  return;
+}
+
+####
+sub _process_error {
+  my __PACKAGE__ $self = shift;
+  my $err_msg = shift;
+
+  my $cmd = shift( @{$self->{processing_queue}} );
+  if ( defined( $cmd ) ) {
+    my $err_code;
+    if ( index( $err_msg, 'LOADING' ) == 0 ) {
+      $err_code = E_LOADING_DATASET;
+    }
+    elsif ( $err_msg eq 'ERR invalid password' ) {
+      $err_code = E_INVALID_PASS;
+    }
+    elsif ( $err_msg eq 'ERR operation not permitted' ) {
+      $err_code = E_AUTH_REQUIRED;
+    }
     else {
-      $self->{on_error}->( "Don't known how process message."
-        . " Unknown channel or pattern '$data->[1]'" );
-      return;
+      $err_code = E_COMMAND_EXEC;
     }
+    $cmd->{on_error}->( $err_msg, $err_code );
   }
-
-  if ( exists( $SUB_ACTION_CMDS{$cmd->{name}} ) ) {
-    return $self->_process_sub_action( $cmd, $data );
-  }
-  shift( @{$self->{processing_queue}} );
-  if ( $cmd->{name} eq 'quit' ) {
-    $self->disconnect();
-  }
-  if ( defined( $cmd->{on_done} ) ) {
-    $cmd->{on_done}->( $data );
+  else {
+    $self->{on_error}->( "Don't known how process error message '$err_msg'."
+      . " Command queue is empty", E_UNEXPECTED );
   }
 
   return;
@@ -624,19 +552,155 @@ sub _process_sub_action {
 }
 
 ####
-sub _process_sub_message {
-  my __PACKAGE__ $self = shift;
-  my $data = shift;
+sub _is_pub_message {
+  my $data = pop;
 
-  my $sub = $self->{subs}{$data->[1]};
-  if ( exists( $sub->{on_message} ) ) {
-    if ( $data->[0] eq 'message' ) {
-      $sub->{on_message}->( $data->[1], $data->[2] );
+  return ( ref( $data ) eq 'ARRAY' and ( $data->[0] eq 'message'
+      or $data->[0] eq 'pmessage' ) );
+}
+
+####
+sub _exec_command {
+  my __PACKAGE__ $self = shift;
+  my $cmd_name = shift;
+  my @args = @_;
+  my $params = {};
+  if ( ref( $args[-1] ) eq 'HASH' ) {
+    $params = pop( @args );
+  }
+
+  $params = $self->_validate_exec_cmd( $params );
+
+  my $cmd = {
+    name => $cmd_name,
+    args => \@args,
+    %{$params},
+  };
+
+  if ( exists( $SUB_ACTION_CMDS{$cmd->{name}} ) ) {
+    if ( $self->{sub_lock} ) {
+      $cmd->{on_error}->( "Command '$cmd->{name}' not allowed after 'multi' command."
+          . ' First, the transaction must be completed', E_COMMAND_EXEC );
+      return;
+    }
+    $cmd->{resp_remaining} = scalar( @{$cmd->{args}} );
+  }
+  elsif ( $cmd->{name} eq 'multi' ) {
+    $self->{sub_lock} = 1;
+  }
+  elsif ( $cmd->{name} eq 'exec' ) {
+    $self->{sub_lock} = 0;
+  }
+
+  if ( !defined( $self->{handle} ) ) {
+    if ( $self->{reconnect} ) {
+      $self->_connect();
     }
     else {
-      $sub->{on_message}->( $data->[2], $data->[3], $data->[1] );
+      $cmd->{on_error}->( "Can't handle the command '$cmd->{name}'."
+          . ' No connection to the server', E_NO_CONN );
+      return;
     }
   }
+  if ( $self->{connected} ) {
+    if ( $self->{authed} ) {
+      $self->_push_to_handle( $cmd );
+    }
+    else {
+      if ( !$self->{authing} ) {
+        $self->_auth();
+      }
+      push( @{$self->{buffer}}, $cmd );
+    }
+  }
+  else {
+    push( @{$self->{buffer}}, $cmd );
+  }
+
+  return;
+}
+
+####
+sub _validate_exec_cmd {
+  my __PACKAGE__ $self = shift;
+  my $params = shift;
+
+  foreach my $name ( qw( on_done on_message on_error ) ) {
+    if (
+      defined( $params->{$name} )
+        and ref( $params->{$name} ) ne 'CODE'
+        ) {
+      croak "'$name' callback must be a code reference";
+    }
+  }
+
+  if ( !defined( $params->{on_error} ) ) {
+    $params->{on_error} = $self->{on_error};
+  }
+
+  return $params;
+}
+
+####
+sub _auth {
+  my __PACKAGE__ $self = shift;
+
+  $self->{authing} = 1;
+  $self->_push_to_handle( {
+    name => 'auth',
+    args => [ $self->{password} ],
+    on_done => sub {
+      $self->{authing} = 0;
+      $self->{authed} = 1;
+      $self->_flush_buffer();
+    },
+
+    on_error => sub {
+      my $err_msg = shift;
+      my $err_code = shift;
+
+      $self->{authing} = 0;
+      $self->_abort_all( $err_msg, $err_code );
+      $self->{on_error}->( $err_msg, $err_code );
+    },
+  } );
+
+  return;
+}
+
+####
+sub _flush_buffer {
+  my __PACKAGE__ $self = shift;
+
+  my @commands = @{$self->{buffer}};
+  $self->{buffer} = [];
+  foreach my $cmd ( @commands ) {
+    $self->_push_to_handle( $cmd );
+  }
+
+  return;
+}
+
+####
+sub _push_to_handle {
+  my __PACKAGE__ $self = shift;
+  my $cmd = shift;
+
+  push( @{$self->{processing_queue}}, $cmd );
+  my $cmd_str = '';
+  my $m_bulk_len = 0;
+  foreach my $token ( $cmd->{name}, @{$cmd->{args}} ) {
+    if ( defined( $token ) and $token ne '' ) {
+      if ( defined( $self->{encoding} ) and is_utf8( $token ) ) {
+        $token = $self->{encoding}->encode( $token );
+      }
+      my $token_len = length( $token );
+      $cmd_str .= "\$$token_len" . EOL . $token . EOL;
+      ++$m_bulk_len;
+    }
+  }
+  $cmd_str = "*$m_bulk_len" . EOL . $cmd_str;
+  $self->{handle}->push_write( $cmd_str );
 
   return;
 }
@@ -644,7 +708,8 @@ sub _process_sub_message {
 ####
 sub _abort_all {
   my __PACKAGE__ $self = shift;
-  my $err = shift;
+  my $err_msg = shift;
+  my $err_code = shift;
 
   $self->{sub_lock} = 0;
   $self->{subs} = {};
@@ -661,19 +726,12 @@ sub _abort_all {
     $self->{processing_queue} = [];
   }
   foreach my $cmd ( @commands ) {
-    $cmd->{on_error}->( "Command '$cmd->{name}' aborted: $err" );
+    $cmd->{on_error}->( "Command '$cmd->{name}' aborted: $err_msg", $err_code );
   }
 
   return;
 }
 
-####
-sub _is_sub_message {
-  my $data = pop;
-  return ( ref( $data ) eq 'ARRAY' and ( $data->[0] eq 'message'
-      or $data->[0] eq 'pmessage' ) );
-}
-
 ####
 sub AUTOLOAD {
   our $AUTOLOAD;
@@ -707,9 +765,7 @@ AnyEvent::Redis::RipeRedis - Non-blocking Redis client with reconnection feature
 =head1 SYNOPSIS
 
   use AnyEvent;
-  use AnyEvent::Redis::RipeRedis;
-
-  my $cv = AnyEvent->condvar();
+  use AnyEvent::Redis::RipeRedis qw( :err_codes );
 
   my $redis = AnyEvent::Redis::RipeRedis->new(
     host => 'localhost',
@@ -721,14 +777,30 @@ AnyEvent::Redis::RipeRedis - Non-blocking Redis client with reconnection feature
       print "Connected\n";
     },
 
+    on_disconnect => sub {
+      print "Disconnected from Redis server\n";
+    },
+
     on_error => sub {
-      my $err = shift;
-      warn "$err\n";
+      my $err_msg = shift;
+      my $err_code = shift;
+
+      warn "$err_msg. Error code: $err_code\n";
     },
   );
 
-  # Set value
-  $redis->set( 'bar', 'Some string', {
+  my $cv = AnyEvent->condvar();
+
+  # Increment
+  $redis->incr( 'foo', {
+    on_done => sub {
+      my $data = shift;
+      print "$data\n";
+    },
+  } );
+
+  # Set value (retry on error)
+  set( $redis, 'bar', 'Some string', {
     on_done => sub {
       my $data = shift;
       print "$data\n";
@@ -736,17 +808,56 @@ AnyEvent::Redis::RipeRedis - Non-blocking Redis client with reconnection feature
     },
 
     on_error => sub {
-      my $err = shift;
-      warn "$err\n";
+      my $err_msg = shift;
+      my $err_code = shift;
+
+      warn "$err_msg. Error code: $err_code\n";
       $cv->croak();
-    },
+    }
   } );
 
   $cv->recv();
 
-=head1 DESCRIPTION
+  $redis->disconnect();
 
-This documentation describes client of version 1.007 and later.
+
+  ####
+  sub set {
+    my $redis = shift;
+    my $key = shift;
+    my $value = shift;
+    my $params = shift;
+
+    $redis->set( $key, $value, {
+      on_done => $params->{on_done},
+      on_error => sub {
+        my $err_msg = shift;
+        my $err_code = shift;
+
+        if (
+          $err_code == E_CANT_CONN
+            or $err_code == E_LOADING_DATASET
+            or $err_code == E_IO_OPERATION
+            or $err_code == E_CONN_CLOSED_BY_REMOTE_HOST
+            ) {
+          warn "$err_msg. Error code: $err_code\n";
+          my $timer;
+          $timer = AnyEvent->timer(
+            after => 3,
+            cb => sub {
+              undef( $timer );
+              set( $redis, $key, $value, $params );
+            },
+          );
+        }
+        else {
+          $params->{on_error}->( $err_msg, $err_code );
+        }
+      },
+    } );
+  }
+
+=head1 DESCRIPTION
 
 AnyEvent::Redis::RipeRedis is a non-blocking Redis client with with reconnection
 feature. It supports subscriptions, transactions, has simple API and it faster
@@ -773,13 +884,17 @@ Requires Redis 1.2 or higher and any supported event loop.
     },
 
     on_connect_error => sub {
-      my $err = shift;
-      warn "$err\n";
+      my $err_msg = shift;
+      my $err_code = shift;
+
+      warn "$err_msg. Error code: $err_code\n";
     },
 
     on_error => sub {
-      my $err = shift;
-      warn "$err\n";
+      my $err_msg = shift;
+      my $err_code = shift;
+
+      warn "$err_msg. Error code: $err_code\n";
     },
   );
 
@@ -843,34 +958,11 @@ resolve the hostname, failure to connect, or a read error.
       my $data = shift;
       print "$data\n";
     },
-
-    on_error => sub {
-      my $err = shift;
-      warn "$err\n";
-    },
   } );
 
   # Set value
   $redis->set( 'bar', 'Some string' );
 
-  # Get value
-  $redis->get( 'bar', {
-    on_done => sub {
-      my $data = shift;
-      print "$data\n";
-    },
-  } );
-
-  # Push values
-  for ( my $i = 1; $i <= 3; $i++ ) {
-    $redis->rpush( 'list', "element_$i", {
-      on_done => sub {
-        my $data = shift;
-        print "$data\n";
-      },
-    } );
-  }
-
   # Get list of values
   $redis->lrange( 'list', 0, -1, {
     on_done => sub {
@@ -881,8 +973,10 @@ resolve the hostname, failure to connect, or a read error.
     },
 
     on_error => sub {
-      my $err = shift;
-      warn "$err\n";
+      my $err_msg = shift;
+      my $err_code = shift;
+
+      warn "$err_msg. Error code: $err_code\n";
     },
   } );
 
@@ -931,8 +1025,8 @@ Subscribe to group of channels by pattern
     },
 
     on_error => sub {
-      my $err = shift;
-      warn "$err\n";
+      my $err_msg = shift;
+      warn "$err_msg\n";
     },
   } );
 
@@ -949,8 +1043,8 @@ Unsubscribe from channel by name
     },
 
     on_error => sub {
-      my $err = shift;
-      warn "$err\n";
+      my $err_msg = shift;
+      warn "$err_msg\n";
     },
   } );
 
@@ -967,8 +1061,8 @@ Unsubscribe from group of channels by pattern
     },
 
     on_error => sub {
-      my $err = shift;
-      warn "$err\n";
+      my $err_msg = shift;
+      warn "$err_msg\n";
     },
   } );
 
@@ -983,6 +1077,69 @@ the parameter C<port> you must specify the path to the socket.
     port => '/tmp/redis.sock',
   );
 
+=head1 ERROR CODES
+
+Error codes were introduced in version of module 1.100
+
+  1  - E_CANT_CONN
+  2  - E_LOADING_DATASET
+  3  - E_IO_OPERATION
+  4  - E_CONN_CLOSED_BY_REMOTE_HOST
+  5  - E_CONN_CLOSED_ON_DEMAND
+  6  - E_NO_CONN
+  7  - E_INVALID_PASS
+  8  - E_AUTH_REQUIRED
+  9  - E_COMMAND_EXEC
+  10 - E_UNEXPECTED
+
+=over
+
+=item E_CANT_CONN
+
+Can't connect to server.
+
+=item E_LOADING_DATASET
+
+Redis is loading the dataset in memory.
+
+=item E_IO_OPERATION
+
+I/O error operation. Connection closed.
+
+=item E_CONN_CLOSED_BY_REMOTE_HOST
+
+Connection closed by remote host.
+
+=item E_CONN_CLOSED_ON_DEMAND
+
+Connection closed on demand.
+
+=item E_NO_CONN
+
+No connection to the server.
+
+=item E_INVALID_PASS
+
+Invalid password
+
+=item E_AUTH_REQUIRED
+
+Operation not permitted. Authentication required.
+
+=item E_COMMAND_EXEC
+
+Command execution error.
+
+=item E_UNEXPECTED
+
+Unexpected error.
+
+=back
+
+To use error codes constants you must import them.
+
+  use AnyEvent::Redis::RipeRedis qw( :err_codes );
+
 =head1 DISCONNECTION FROM SERVER
 
 When the connection to the server is no longer needed you can close it in three
@@ -1012,11 +1169,17 @@ Eugene Ponizovsky, E<lt>ponizovsky@gmail.comE<gt>
 
 =over
 
-=item Alexey Shrub
+=item *
 
-=item Vadim Vlasov
+Alexey Shrub
 
-=item Konstantin Uvarin
+=item *
+
+Vadim Vlasov
+
+=item *
+
+Konstantin Uvarin
 
 =back
 
